@@ -27,6 +27,7 @@
 #include "config.h"
 #include "asc-font-private.h"
 
+#include <glib/gstdio.h>
 #include <fontconfig/fontconfig.h>
 #include FT_SFNT_NAMES_H
 #include FT_TRUETYPE_IDS_H
@@ -335,13 +336,22 @@ asc_font_new_from_data (const void *data, gssize len, const gchar *file_basename
 	 * (FreeType itself could load from memory) */
 	tmp_root = asc_globals_get_tmp_dir_create ();
 
-        fname = g_build_filename(tmp_root, file_basename, NULL);
+        fname = g_build_filename (tmp_root, file_basename, NULL);
+#if GLIB_CHECK_VERSION(2,66,0)
 	ret = g_file_set_contents_full (fname,
 					data,
 					len,
 					G_FILE_SET_CONTENTS_NONE,
 					0666,
 					error);
+#else
+	ret = g_file_set_contents (fname,
+				   data,
+				   len,
+				   error);
+	if (ret)
+		g_chmod (fname, 0666);
+#endif
 	if (!ret)
 		return NULL;
 
@@ -673,10 +683,11 @@ asc_font_determine_sample_texts (AscFont *font)
 
 	/* If we only have to set the icon text, try to do it!
 	 * Otherwise keep cached values and do nothing */
-        if (!as_is_empty (priv->sample_text))
-            asc_font_set_fallback_sample_texts_if_needed (font);
-        if (!as_is_empty (priv->sample_icon_text))
-            return;
+        if (!as_is_empty (priv->sample_text)) {
+		asc_font_set_fallback_sample_texts_if_needed (font);
+		if (!as_is_empty (priv->sample_icon_text))
+			return;
+	}
 
 	/* always prefer English (even if not alphabetically first) */
 	if (g_hash_table_contains (priv->languages, "en")) {
@@ -708,13 +719,16 @@ asc_font_determine_sample_texts (AscFont *font)
 
 	/* check if we have a font that can actually display the characters we picked - in case
 	 * it doesn't, we just select random chars. */
-	if (FT_Get_Char_Index (priv->fface, g_utf8_get_char_validated (priv->sample_icon_text, -1)) == 0) {
+	if (FT_Get_Char_Index (priv->fface, g_utf8_get_char_validated (priv->sample_icon_text, -1)) != 0)
+		return;
+
+	if (FT_Get_Char_Index (priv->fface, g_utf8_get_char_validated ("☃", -1)) != 0) {
+		/* maybe we have a symbols-only font? */
 		g_free (priv->sample_text);
 		g_free (priv->sample_icon_text);
 		priv->sample_text = g_strdup ("☃❤✓☀★☂♞☯☢∞❄♫↺");
 		priv->sample_icon_text = g_strdup ("☃❤");
-	}
-	if (FT_Get_Char_Index (priv->fface, g_utf8_get_char_validated (priv->sample_icon_text, -1)) == 0) {
+	} else {
 		GString *sample_text;
 		guint count;
 
