@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2014-2015 Matthias Klumpp <matthias@tenstral.net>
+ * Copyright (C) 2014-2020 Matthias Klumpp <matthias@tenstral.net>
  *
  * Licensed under the GNU Lesser General Public License Version 2.1
  *
@@ -60,11 +60,23 @@ typedef enum {
 	AS_IMAGE_KIND_LAST
 } AsImageKind;
 
+/* some useful constants */
+#define AS_IMAGE_LARGE_HEIGHT		423	/* Since: 0.14.0 */
+#define AS_IMAGE_LARGE_WIDTH		752	/* Since: 0.14.0 */
+#define AS_IMAGE_NORMAL_HEIGHT		351	/* Since: 0.14.0 */
+#define AS_IMAGE_NORMAL_WIDTH		624	/* Since: 0.14.0 */
+#define AS_IMAGE_THUMBNAIL_HEIGHT	63	/* Since: 0.14.0 */
+#define AS_IMAGE_THUMBNAIL_WIDTH 	112	/* Since: 0.14.0 */
+
 AsImageKind	 as_image_kind_from_string (const gchar *kind);
 const gchar	*as_image_kind_to_string (AsImageKind kind);
 
 AsImage		*as_image_new (void);
 
+AsImageKind	 as_image_get_kind (AsImage *image);
+void		 as_image_set_kind (AsImage *image,
+					AsImageKind kind);
+
 const gchar	*as_image_get_url (AsImage *image);
 void		 as_image_set_url (AsImage *image,
 					const gchar *url);
@@ -77,9 +89,9 @@ guint		 as_image_get_height (AsImage *image);
 void		 as_image_set_height (AsImage *image,
 					guint height);
 
-AsImageKind	 as_image_get_kind (AsImage *image);
-void		 as_image_set_kind (AsImage *image,
-					AsImageKind kind);
+const gchar	*as_image_get_locale (AsImage *image);
+void		 as_image_set_locale (AsImage *image,
+				      const gchar *locale);
 
 G_END_DECLS
 
