@@ -65,7 +65,8 @@ G_BEGIN_DECLS
  * @AS_TAG_CONTENT_RATING:		`content_rating` / `ContentRating`
  * @AS_TAG_RECOMMENDS:			`recommends` / `Recommends`
  * @AS_TAG_REQUIRES:			`requires` / `Requires`
- * @AS_TAG_AGREEMENT:			`agreement` / `Àgreement`
+ * @AS_TAG_AGREEMENT:			`agreement` / `Agreement`
+ * @AS_TAG_REVIEWS:			`reviews` / `Reviews`
  * @AS_TAG_NAME_VARIANT_SUFFIX:		`name_variant_suffix` / `NameVariantSuffix`
  * @AS_TAG_P:				Description markup `p`
  * @AS_TAG_LI:				Description markup `li`
@@ -109,6 +110,7 @@ typedef enum {
 	AS_TAG_RECOMMENDS,
 	AS_TAG_REQUIRES,
 	AS_TAG_AGREEMENT,
+	AS_TAG_REVIEWS,
 	AS_TAG_NAME_VARIANT_SUFFIX,
 	AS_TAG_P,
 	AS_TAG_LI,
@@ -117,6 +119,7 @@ typedef enum {
 
 	AS_TAG_INTERNAL_SCOPE,
 	AS_TAG_INTERNAL_ORIGIN,
+	AS_TAG_INTERNAL_BRANCH,
 
 	/*< private >*/
 	AS_TAG_LAST
