@@ -34,7 +34,7 @@
  * as_xml_get_node_value:
  */
 gchar*
-as_xml_get_node_value (xmlNode *node)
+as_xml_get_node_value (const xmlNode *node)
 {
 	gchar *content;
 	content = (gchar*) xmlNodeGetContent (node);
@@ -44,6 +44,56 @@ as_xml_get_node_value (xmlNode *node)
 	return content;
 }
 
+/**
+ * as_xml_get_node_value_refstr:
+ */
+GRefString*
+as_xml_get_node_value_refstr (const xmlNode *node)
+{
+	g_autofree gchar *content = NULL;
+	content = (gchar*) xmlNodeGetContent (node);
+	if (content != NULL)
+		as_strstripnl (content);
+	if (content == NULL)
+		return g_ref_string_new_intern ("");
+	return g_ref_string_new_intern (content);
+}
+
+/**
+ * as_xml_get_prop_value_refstr:
+ */
+GRefString*
+as_xml_get_prop_value_refstr (const xmlNode *node, const gchar *prop_name)
+{
+	g_autofree gchar *tmp = as_xml_get_prop_value (node, prop_name);
+	return g_ref_string_new_intern (tmp);
+}
+
+/**
+ * as_xml_get_prop_value_as_int:
+ *
+ * Gets a XML node property, e.g. 34
+ *
+ * Return value: integer value, or %G_MAXINT for error
+ **/
+gint
+as_xml_get_prop_value_as_int (const xmlNode *node, const gchar *prop_name)
+{
+	g_autofree gchar *tmp = NULL;
+	gchar *endptr = NULL;
+	gint64 value_tmp;
+
+	tmp = as_xml_get_prop_value (node, prop_name);
+	if (tmp == NULL)
+		return G_MAXINT;
+	value_tmp = g_ascii_strtoll (tmp, &endptr, 10);
+	if (value_tmp == 0 && tmp == endptr)
+		return G_MAXINT;
+	if (value_tmp > G_MAXINT || value_tmp < G_MININT)
+		return G_MAXINT;
+	return (gint) value_tmp;
+}
+
 /**
  * as_xml_get_node_locale_match:
  * @node: A XML node
@@ -280,7 +330,7 @@ as_xml_markup_parse_helper_new (const gchar *markup, const gchar *locale)
 	AsXMLMarkupParseHelper *helper = g_slice_new0 (AsXMLMarkupParseHelper);
 
 	helper->locale = g_strdup (locale);
-	xmldata = g_strdup_printf ("<root>%s</root>", markup);
+	xmldata = g_strconcat ("<root>", markup, "</root>", NULL);
 	helper->doc = xmlReadMemory (xmldata, strlen (xmldata),
 					NULL,
 					"utf-8",
@@ -374,18 +424,50 @@ as_xml_markup_parse_helper_export_node (AsXMLMarkupParseHelper *helper, xmlNode
 	return NULL;
 }
 
+typedef struct
+{
+	guint elem_count;
+	GString *data;
+} AsXMLMetaInfoDescParseHelper;
+
+/**
+ * as_xml_metainfo_desc_parse_helper_new: (skip)
+ **/
+static AsXMLMetaInfoDescParseHelper*
+as_xml_metainfo_desc_parse_helper_new ()
+{
+	AsXMLMetaInfoDescParseHelper *helper = g_slice_new0 (AsXMLMetaInfoDescParseHelper);
+	helper->data = g_string_new ("");
+	helper->elem_count = 0;
+	return helper;
+}
+
+/**
+ * as_xml_metainfo_desc_parse_helper_free: (skip)
+ **/
+static gchar*
+as_xml_metainfo_desc_parse_helper_free (AsXMLMetaInfoDescParseHelper *helper)
+{
+	gchar *data = g_string_free (helper->data, FALSE);
+	g_slice_free (AsXMLMetaInfoDescParseHelper, helper);
+	return data;
+}
+
 /**
  * as_xml_parse_metainfo_description_node:
  */
 void
-as_xml_parse_metainfo_description_node (AsContext *ctx, xmlNode *node, GHFunc func, gpointer entity)
+as_xml_parse_metainfo_description_node (AsContext *ctx, xmlNode *node, GHashTable *l10n_desc)
 {
-	xmlNode *iter;
-	g_autoptr(GHashTable) desc = NULL;
+	g_autoptr(GHashTable) tmp_desc = NULL;
+	GHashTableIter res_iter;
+	gpointer res_value;
+	gpointer res_key;
+	AsXMLMetaInfoDescParseHelper *phelper;
+	guint untranslated_elem_count = 0;
 
-	desc = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
-	for (iter = node->children; iter != NULL; iter = iter->next) {
-		GString *str;
+	tmp_desc = g_hash_table_new_full (g_str_hash, g_str_equal, (GDestroyNotify) g_ref_string_release, NULL);
+	for (xmlNode *iter = node->children; iter != NULL; iter = iter->next) {
 		AsTag tag_id;
 		const gchar *node_name = (const gchar*) iter->name;
 
@@ -403,15 +485,19 @@ as_xml_parse_metainfo_description_node (AsContext *ctx, xmlNode *node, GHFunc fu
 				/* this locale is not for us */
 				continue;
 
-			str = g_hash_table_lookup (desc, lang);
-			if (str == NULL) {
-				str = g_string_new ("");
-				g_hash_table_insert (desc, g_strdup (lang), str);
+			phelper = g_hash_table_lookup (tmp_desc, lang);
+			if (phelper == NULL) {
+				phelper = as_xml_metainfo_desc_parse_helper_new ();
+				g_hash_table_insert (tmp_desc,
+						     g_ref_string_new_intern (lang),
+						     phelper);
 			}
 
 			content = as_xml_dump_desc_para_node_content_raw (iter);
-			if (content != NULL)
-				g_string_append_printf (str, "<p>%s</p>\n", content);
+			if (content != NULL) {
+				g_string_append_printf (phelper->data, "<p>%s</p>\n", content);
+				phelper->elem_count += 1;
+			}
 
 		} else if ((tag_id == AS_TAG_UL) || (tag_id == AS_TAG_OL)) {
 			GHashTableIter htiter;
@@ -419,9 +505,9 @@ as_xml_parse_metainfo_description_node (AsContext *ctx, xmlNode *node, GHFunc fu
 			xmlNode *iter2;
 
 			/* append listing node tag to every locale string */
-			g_hash_table_iter_init (&htiter, desc);
+			g_hash_table_iter_init (&htiter, tmp_desc);
 			while (g_hash_table_iter_next (&htiter, NULL, &hvalue)) {
-				GString *hstr = (GString*) hvalue;
+				GString *hstr = ((AsXMLMetaInfoDescParseHelper*) hvalue)->data;
 				g_string_append_printf (hstr, "<%s>\n", node_name);
 			}
 
@@ -440,28 +526,61 @@ as_xml_parse_metainfo_description_node (AsContext *ctx, xmlNode *node, GHFunc fu
 					continue;
 
 				/* if the language is new, we add a listing tag first */
-				str = g_hash_table_lookup (desc, lang);
-				if (str == NULL) {
-					str = g_string_new ("");
-					g_string_append_printf (str, "<%s>\n", node_name);
-					g_hash_table_insert (desc, g_strdup (lang), str);
+				phelper = g_hash_table_lookup (tmp_desc, lang);
+				if (phelper == NULL) {
+					phelper = as_xml_metainfo_desc_parse_helper_new ();
+					g_string_append_printf (phelper->data, "<%s>\n", node_name);
+					g_hash_table_insert (tmp_desc,
+							     g_ref_string_new_intern (lang),
+							     phelper);
 				}
 
 				content = as_xml_dump_desc_para_node_content_raw (iter2);
-				if (content != NULL)
-					g_string_append_printf (str, "  <%s>%s</%s>\n", (gchar*) iter2->name, content, (gchar*) iter2->name);
+				if (content != NULL) {
+					g_string_append_printf (phelper->data, "  <%s>%s</%s>\n",
+								(gchar*) iter2->name,
+								content,
+								(gchar*) iter2->name);
+					phelper->elem_count += 1;
+				}
 			}
 
 			/* close listing tags */
-			g_hash_table_iter_init (&htiter, desc);
+			g_hash_table_iter_init (&htiter, tmp_desc);
 			while (g_hash_table_iter_next (&htiter, NULL, &hvalue)) {
-				GString *hstr = (GString*) hvalue;
+				GString *hstr = ((AsXMLMetaInfoDescParseHelper*) hvalue)->data;
 				g_string_append_printf (hstr, "</%s>\n", node_name);
 			}
 		}
 	}
 
-	g_hash_table_foreach (desc, func, entity);
+	phelper = g_hash_table_lookup (tmp_desc, "C");
+	if (phelper != NULL)
+		untranslated_elem_count = phelper->elem_count;
+
+	/* finalize the data */
+	g_hash_table_iter_init (&res_iter, tmp_desc);
+	while (g_hash_table_iter_next (&res_iter, &res_key, &res_value)) {
+		g_autofree gchar *text = NULL;
+		guint elem_count;
+		phelper = (AsXMLMetaInfoDescParseHelper*) res_value;
+
+		elem_count = phelper->elem_count;
+		text = as_xml_metainfo_desc_parse_helper_free (phelper);
+
+		/* we require at the very least either more than 3 elements of the description to be translated or
+		 * all of the elements if there are less than 3 elements to accept a translation.
+		 * See https://github.com/ximion/appstream/issues/293 for more information on the kind of issue that
+		 * caused this workaround. */
+		if (elem_count < 3) {
+			if (elem_count < untranslated_elem_count)
+				continue;
+		}
+
+		g_hash_table_insert (l10n_desc,
+				     g_ref_string_acquire (res_key),
+				     g_steal_pointer (&text));
+	}
 }
 
 /**
@@ -616,6 +735,47 @@ as_xml_add_description_node (AsContext *ctx, xmlNode *root, GHashTable *desc_tab
 	}
 }
 
+/**
+ * as_xml_add_description_node_raw:
+ *
+ * Add a simple description node in verbatim, performing only basic markup
+ * validation. The node will not have a language property attached.
+ *
+ * Returns: The new xmlNode, or %NULL if no node was appended.
+ */
+xmlNode*
+as_xml_add_description_node_raw (xmlNode *root, const gchar *description)
+{
+	xmlNode *dnode;
+	xmlNode *cnode;
+	g_autoptr(AsXMLMarkupParseHelper) helper = NULL;
+
+	if (as_is_empty (description))
+		return NULL;
+
+	helper = as_xml_markup_parse_helper_new (description, NULL);
+	if (helper == NULL)
+		return NULL;
+
+	dnode = xmlNewChild (root, NULL, (xmlChar*) "description", NULL);
+	if (helper->node == NULL)
+		return NULL;
+	cnode = dnode;
+
+	do {
+		if ((helper->tag_id == AS_TAG_UL) || (helper->tag_id == AS_TAG_OL)) {
+			cnode = as_xml_markup_parse_helper_export_node (helper, dnode, FALSE);
+		} else {
+			if (helper->tag_id != AS_TAG_LI)
+				cnode = dnode;
+
+			as_xml_markup_parse_helper_export_node (helper, cnode, FALSE);
+		}
+	} while (as_xml_markup_parse_helper_next (helper));
+
+	return dnode;
+}
+
 /**
  * as_xml_add_localized_text_node:
  *
@@ -718,9 +878,70 @@ as_xml_add_node_list (xmlNode *root, const gchar *name, const gchar *child_name,
 }
 
 /**
- * as_xml_add_text_node:
+ * as_xml_parse_custom_node:
  *
- * Add node if value is not empty
+ * Parse a custom key/value table from XML into a #GHashTable
+ * using #GRefString as key/value.
+ */
+void
+as_xml_parse_custom_node (xmlNode *node, GHashTable *custom)
+{
+	for (xmlNode *iter = node->children; iter != NULL; iter = iter->next) {
+		g_autofree gchar *key_str = NULL;
+
+		if (iter->type != XML_ELEMENT_NODE)
+			continue;
+		if (g_strcmp0 ((gchar*) iter->name, "value") != 0)
+			continue;
+
+		key_str = (gchar*) xmlGetProp (iter, (xmlChar*) "key");
+		if (key_str == NULL)
+			continue;
+
+		g_hash_table_insert (custom,
+				     g_ref_string_new_intern (key_str),
+				     as_xml_get_node_value_refstr (iter));
+	}
+}
+
+/**
+ * as_xml_add_custom_node:
+ *
+ * Add a custom key/value table to the XML DOM.
+ * The #GHashTable should use #GRefString as keys/values.
+ */
+void
+as_xml_add_custom_node (xmlNode *root, const gchar *node_name, GHashTable *custom)
+{
+	xmlNode *node;
+	g_autoptr(GList) keys = NULL;
+
+	if (g_hash_table_size (custom) == 0)
+		return;
+
+	node = xmlNewChild (root, NULL, (xmlChar*) node_name, NULL);
+	keys = g_hash_table_get_keys (custom);
+	keys = g_list_sort (keys, (GCompareFunc) g_ascii_strcasecmp);
+	for (GList *link = keys; link != NULL; link = link->next) {
+		const GRefString *key = (const GRefString*) link->data;
+
+		xmlNode *snode = xmlNewTextChild (node,
+						  NULL,
+						  (xmlChar*) "value",
+						  (xmlChar*) g_hash_table_lookup (custom, key));
+		xmlNewProp (snode,
+			    (xmlChar*) "key",
+			    (xmlChar*) key);
+	}
+}
+
+/**
+ * as_xml_add_text_node:
+ * @root: The node to add a child to.
+ * @name: The new node name.
+ * @value: The new node value.
+ *
+ * Add node if value is not empty.
  */
 xmlNode*
 as_xml_add_text_node (xmlNode *root, const gchar *name, const gchar *value)
@@ -731,6 +952,23 @@ as_xml_add_text_node (xmlNode *root, const gchar *name, const gchar *value)
 	return xmlNewTextChild (root, NULL, (xmlChar*) name, (xmlChar*) value);
 }
 
+/**
+ * as_xml_add_text_prop:
+ * @node: The node to attach a property to.
+ * @name: The new property name.
+ * @value: The new property value.
+ *
+ * Add property to node if value is not empty.
+ */
+xmlAttr*
+as_xml_add_text_prop (xmlNode *node, const gchar *name, const gchar *value)
+{
+	if (as_is_empty (value))
+		return NULL;
+
+	return xmlNewProp (node, (xmlChar*) name, (xmlChar*) value);
+}
+
 /**
  * libxml_generic_error:
  *
@@ -805,12 +1043,12 @@ as_xml_parse_document (const gchar *data, gssize len, GError **error)
 		if (error_msg_str == NULL) {
 			g_set_error (error,
 					AS_METADATA_ERROR,
-					AS_METADATA_ERROR_FAILED,
+					AS_METADATA_ERROR_PARSE,
 					"Could not parse XML data (no details received)");
 		} else {
 			g_set_error (error,
 					AS_METADATA_ERROR,
-					AS_METADATA_ERROR_FAILED,
+					AS_METADATA_ERROR_PARSE,
 					"Could not parse XML data: %s", error_msg_str);
 		}
 		as_xml_set_out_of_context_error (NULL);
@@ -822,7 +1060,7 @@ as_xml_parse_document (const gchar *data, gssize len, GError **error)
 	if (root == NULL) {
 		g_set_error_literal (error,
 				     AS_METADATA_ERROR,
-				     AS_METADATA_ERROR_FAILED,
+				     AS_METADATA_ERROR_PARSE,
 				     "The XML document is empty.");
 		xmlFreeDoc (doc);
 		return NULL;
